#!/usr/bin/perl
#################################### Modules ##########################################
# created by markus@nikulski.net

our $VERSION = v0.0.1;
require 5.0.6;
use strict;
use warnings;
use IO::Socket::INET;
use Time::HiRes qw(usleep);

#################################### init Variables ###################################
my $sock;
my %ripCommand = (
						Request		=> '1',																			# A request for the responding system to send all or part of its routing table
						Response	=> '2',																			# A message containing all or part of the senders routing table. This message may be sent in response to a request or poll, or it may be an update message generated by the sender.
						Traceon		=> '3',																			# Obsolete
						Traceoff	=> '4',																			# Obsolete
						Reserved	=> '5');																		# Used by Sun Microsystems for its own purposes.
my $ripVersion				= '2';
my $RoutingDomain			= '00';
my $ripAddressFamily	= '02';																			# Indicates what type of address is specified in this particular entry. This is used because RIP2 may carry routing information for several different protocols. The address family identifier for IP is 2.
my $delayPerPDU		 		= 0;																				# delay between updates
my $routeLimit				= 100_000;
my @ipAddressStore;
my %cfg = (
			routeCount		=> 1,
			metric				=> 15,
			nextHop				=> '0.0.0.0',
			ripRouteTag		=> 0,
			updateDelay		=> 30,
			routesPerPDU	=> 25,
			delayPerPDU		=> 0,
			updates				=> 'endless',
			default				=> 0,
		);

#######################################################################################
#################################### Main #############################################
#######################################################################################

initArguments();
initRip();
genrateRoutes();
sendRoutes();

#######################################################################################
################################ Subroutines ##########################################
#######################################################################################
#initArguments ();
sub initArguments{
	my $startUp = $0;
	$startUp =~ /RIPgen.[\w]+$|ripgen.[\w]+$/;
	$startUp = $&;
	my ($syntax) = <<EOFinfo;
RIP route generator/simulator
Usage: $startUp -n <IPnet/mask> [options]

Parameter:
  -n <IPnet/mask>	initial network with mask

Option:
  -a <count>	amount of generate routes\t(default 1)
  -m <metric>	metric of all RIP routes\t(default 15)
  -h <ip>	next hop of all RIP routes\t(default 0.0.0.0)
  -t <tag>	tag of all RIP routes\t\t(default 0)
  -d <second>	delay between RIP updates\t(default 30)
  -r <count>	routes per RIP PDU\t\t(default 25)
  -dp <msecond>	delay between PDUs\t\t(default 0)
  -u <count>	amount of RIP updates\t\t(default endless)
  -default	generate a default route

Examples:
  $startUp -n 10.1.1.0/24
  $startUp -n 10.1.1.0/255.255.255.0 -a 120
  $startUp -n 10.1.1.0/24 -a 120 -m 3 -d 0 -r 2 -u 15 -h 172.16.1.5
EOFinfo
	
	STDOUT->autoflush(1);

	if(!defined($ARGV[0])){
		die $syntax;
	}else{
		my $count = 0;
		foreach my $in (@ARGV){
			$cfg{'net'}			= $ARGV[$count + 1] || 0										if($in =~ /^-n$/);
			$cfg{'routeCount'}	= $ARGV[$count + 1] || $cfg{'routeCount'}		if($in =~ /^-a$/);
			$cfg{'metric'}			= $ARGV[$count + 1] || $cfg{'metric'}				if($in =~ /^-m$/);
			$cfg{'nextHop'}			= $ARGV[$count + 1] || $cfg{'nextHop'}			if($in =~ /^-h$/);
			$cfg{'ripRouteTag'}	= $ARGV[$count + 1] || $cfg{'ripRouteTag'}	if($in =~ /^-t$/);
			$cfg{'updateDelay'}	= $ARGV[$count + 1] || $cfg{'updateDelay'}	if($in =~ /^-d$/);
			$cfg{'routesPerPDU'}= $ARGV[$count + 1] || $cfg{'routesPerPDU'}	if($in =~ /^-r$/);
			$cfg{'delayPerPDU'}	= $ARGV[$count + 1] || $cfg{'delayPerPDU'}	if($in =~ /^-dp$/);
			$cfg{'updates'}			= $ARGV[$count + 1] || $cfg{'updates'}			if($in =~ /^-u$/);
			$cfg{'default'}			= 1																					if($in =~ /^-default$/);
			$count++;
		}
		
		if(!$cfg{'net'}){
			die "\nERROR: missing required parameter\n\n$syntax";
		}else{
			die "IP network and mask seperator '/' not found\n\n$syntax" if $cfg{'net'} !~ /\//;
			($cfg{'net'}, $cfg{'mask'}) = split /\//, $cfg{'net'};
			$cfg{'mask'} = mask2bin($cfg{'mask'}) if $cfg{'mask'} !~ /^(\d{1,3}\.){3}\d{1,3}/;
			die "IP network '".$cfg{'net'}."' is wrong\n\n$syntax" if $cfg{'net'} !~ /^(\d{1,3}\.){3}\d{1,3}/;
			die "IP mask '".$cfg{'mask'}."'is wrong\n\n$syntax" if $cfg{'mask'} !~ /^(\d{1,3}\.){3}\d{1,3}/;
			die "amount of requested routes exite $routeLimit\n" if $cfg{'routeCount'} > $routeLimit;
			die "amount of routes per PDU doesn't fit in a Ethernet frame!\n" if $cfg{'routesPerPDU'} > 73;
			print "\nWARINING: amount of routes per PDU isn't RFC conform!\n\n" if $cfg{'routesPerPDU'} > 25;
			$cfg{'delayPerPDU'} = $cfg{'delayPerPDU'} * 1000 if $cfg{'delayPerPDU'};
		}
		
		$cfg{'metric'} =~ s/([0-9]+)/sprintf("%x", $1)/ge;
		$cfg{'metric'} = '0'.$cfg{'metric'} while length($cfg{'metric'}) < 4;
		
		$cfg{'ripRouteTag'} = '0'.$cfg{'ripRouteTag'} while length($cfg{'ripRouteTag'}) < 2;
	}
}

#######################################################################################
#initRip
sub initRip {
	$sock = IO::Socket::INET->new(PeerAddr	=> '192.168.112.1',
																LocalPort => 520,
																PeerAddr	=> '224.0.0.9:520',
																Proto			=> 'udp')
		or die "can't init RIP interface!\n";
	
	$sock->blocking(0);
}

#######################################################################################
#genrateRoutes()
sub genrateRoutes {
	my @netDigits = split /\./, $cfg{'net'};
	my @maskDigits = split /\./, $cfg{'mask'};
	my ($stepLength, $checkHostDigit, $digitCount) = findStepDistance(\@maskDigits);
	
	validateNetwork(\@netDigits, \@maskDigits, $checkHostDigit, $stepLength);
	
	push(@ipAddressStore, $cfg{'net'});
	$cfg{'routeCount'}--;
	
	for(my $count = $cfg{'routeCount'}; $count; $count--){
		$netDigits[$digitCount] = $netDigits[$digitCount] + $stepLength;
		if($netDigits[$digitCount] > 255){
			$netDigits[$digitCount] = 0;
			if($digitCount - 1 >= 0){
				$netDigits[$digitCount - 1]++;
				if($netDigits[$digitCount - 1] > 255){
					$netDigits[$digitCount - 1] = 0;
					if($digitCount - 2 >= 0){
						$netDigits[$digitCount - 2]++;
						if($netDigits[$digitCount - 2] > 255){
							$netDigits[$digitCount - 2] = 0;
							if($digitCount - 3 >= 0){
								$netDigits[$digitCount - 3]++;
								if($netDigits[$digitCount - 3] > 255){
									die "no more IP networks available, max ".$cfg{'routeCount'}." possible\n";
								}
							}else{
								die "no more IP networks available, max ".$cfg{'routeCount'}." possible\n";
							}
						}
					}else{
						die "no more IP networks available, max ".$cfg{'routeCount'}." possible\n";
					}
				}
			}else{
				die "no more IP networks available, max ".$cfg{'routeCount'}." possible\n";
			}
		}
		
		push(@ipAddressStore, $netDigits[0].'.'.$netDigits[1].'.'.$netDigits[2].'.'.$netDigits[3]);
		
	}
	
	my $message = '';
	if($cfg{'default'}){
		unshift(@ipAddressStore, '0.0.0.0');
		$message = ' + 0.0.0.0/0';
	}
	
	print $cfg{'net'}.' => '.$netDigits[0].'.'.$netDigits[1].'.'.$netDigits[2].'.'.$netDigits[3]."$message\n";
}

#######################################################################################
#findStepDistance()
sub findStepDistance {
	my $refH_maskDigits = shift;
	my $checkHostDigit = 4;
	my $stepDistance;
	my $digitCount = 0;
	
	foreach (@$refH_maskDigits){
		if ($_ < 255){
			$stepDistance = $_;
			last;
		}
		$digitCount++;
	}
	
	if(!defined($stepDistance)){
		$stepDistance = 255;
		$digitCount--;
		$checkHostDigit--;
	}elsif($stepDistance == 0){
		$stepDistance = 255;
		$digitCount--;
		$checkHostDigit--;
	}
	
	$stepDistance = 255 - $stepDistance + 1;
	$checkHostDigit = $checkHostDigit - $digitCount;
	
	return $stepDistance, $checkHostDigit, $digitCount;
}

#######################################################################################
#validateNetwork()
sub validateNetwork {
	my $refH_netDigits = shift;
	my $refH_maskDigits = shift;
	my $checkHostDigit = shift;
	my $stepLength = shift;
	
	for(my $count = 1; $count <= $checkHostDigit; $count++){
		if($$refH_netDigits[-$count] == 0){
			next;
		}else{
			my $calc1 = $$refH_netDigits[-$count] / $stepLength;
			my $calc2 = sprintf "%.0f", $calc1;
			if($calc1 != $calc2){
				die "IP network/mask combination not valid\n";
			}
		}
	}
}

#######################################################################################
#sendRoutes()
sub sendRoutes {
	my @ips;
	my $countPDU;
	my $sendPDUcount = 1;
	my $allRoutes = @ipAddressStore;
		
	$countPDU = sprintf "%.0f", $allRoutes / $cfg{'routesPerPDU'};
	$countPDU++ if $countPDU != $allRoutes / $cfg{'routesPerPDU'};
	my $message = "send $allRoutes routes in $countPDU PDUs\n";
	$message =~ s/PDUs/PDU/ if $countPDU == 1;
	$message =~ s/routes/route/ if $allRoutes == 1;
	print $message;
	
	do{
		$allRoutes = 0;
		splice(@ips);
		my $countDowPDU = $countPDU;
		my $countRoutesPerPDU = $cfg{'routesPerPDU'};
		foreach(@ipAddressStore){
			push(@ips, $_);
			if($countRoutesPerPDU <= 1){
				sendUpdate(@ips);
				splice(@ips);
				$countRoutesPerPDU = $cfg{'routesPerPDU'};
				$countDowPDU--;
			}else{
				$countRoutesPerPDU--;
			}
			$allRoutes++;
		}
		
		sendUpdate(@ips) if $countDowPDU;
		$sendPDUcount++;
		
		if(($cfg{'updates'} eq 'endless') or ($sendPDUcount <= $cfg{'updates'})){
			my $countdown = $cfg{'updateDelay'};
			do{
				print "\rnext RIP update $sendPDUcount/".$cfg{'updates'}." in $countdown seconds ";
				sleep(1);
				$countdown--;
			}while($countdown)
		}
		
	}while(($cfg{'updates'} eq 'endless') or ($sendPDUcount <= $cfg{'updates'}));
	print "\r\t\t\t\t\n";
}

#######################################################################################
#sendUpdate
sub sendUpdate {
	my $ripFrameHeader = dec2unicode($ripCommand{'Response'}.$ripVersion.$RoutingDomain);
	my $ripRouteHeader = dec2unicode($ripAddressFamily.$cfg{'ripRouteTag'});
	my $ripRouteEntries = '';
	my $count = 0;
	
	foreach my $ip (@_){
		my $mask = $cfg{'mask'};
		$mask = '0.0.0.0' if $ip eq '0.0.0.0';
		$ripRouteEntries .= $ripRouteHeader.inet_aton($ip).inet_aton($mask).inet_aton($cfg{'nextHop'}).dec2unicode($cfg{'metric'});
		$count++;
	}
	
	$sock->send($ripFrameHeader.$ripRouteEntries);
	
	if($cfg{'delayPerPDU'}){
		print "\rdelaying PDU                           ";
		usleep($cfg{'delayPerPDU'});
	}
}

#######################################################################################
#dec2unicode(<"decimal integer">)
sub dec2unicode {
	my $output = '';
	
	foreach my $i (split //, shift){
		next if $i eq '.';
		$output .= pack("h", $i);
	}
	return $output;
}

#######################################################################################
# convert 4dot notation to bit lenth or the other way around
#mask2bin ( $string);
sub mask2bin{
	my $mask = shift;
	my @maskSteps = qw(0 128 192 224 240 248 252 254 255);
	my $out;
	
	if ($mask =~ /^(\d{1,3}\.){3}\d{1,3}$/) {															# 4 dot notation
		$out = 0;
		my @maskParts = split(/\./, $mask);
		foreach my $dot(@maskParts){
			my $count = 0;
			foreach my $bit(@maskSteps) {
				$out = $out + $count if $dot =~ /$bit/;												# add bit from dot
				$count++;
			}
		}
	}elsif (!(($mask < 0) or ($mask > 32))){															# in bit lenth
		my $count = 0;
		if (($mask > 7) and ($mask < 32)) {
			$out = $out."255.";																								# leading full dot
			$mask = $mask - 8;
			while ($mask > 8) {																								# find next full dot
				$out = $out."255.";																							# next full dot
				$mask = $mask - 8;
			}
		}elsif ($mask == 32) {
			return "255.255.255.255";
		}
		$out = $out.$maskSteps[$mask];
		while (!($out =~ /^(\d{1,3}\.){3}\d{1,3}$/)) {											# if mask complete?
			$out = $out.".0";
		}
	}else{
		die "mask '$mask' mailformed \n";
	}
	
	return $out;
}
